@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -361,7 +361,7 @@ def _check_option_defaults(options):
     """
 
     for opt_name in _DEFAULT_OPTIONS:
-        if not opt_name in options:
+        if opt_name not in options:
             options[opt_name] = _DEFAULT_OPTIONS[opt_name]
 
 
@@ -438,6 +438,10 @@ def database_compare(server1_val, server2_val, db1, db2, options):
     reporter = _CompareDBReport(options)
     reporter.print_heading()
 
+    # Get sql_mode value from servers
+    server1_sql_mode = server1.select_variable("SQL_MODE")
+    server2_sql_mode = server2.select_variable("SQL_MODE")
+
     # Remaining operations can occur in a loop one for each object.
     for item in in_both:
         error_list = []
@@ -445,10 +449,12 @@ def database_compare(server1_val, server2_val, db1, db2, options):
         # Set the object type
         obj_type = item[0]
 
-        q_obj1 = "{0}.{1}".format(quote_with_backticks(db1),
-                                  quote_with_backticks(item[1][0]))
-        q_obj2 = "{0}.{1}".format(quote_with_backticks(db2),
-                                  quote_with_backticks(item[1][0]))
+        q_obj1 = "{0}.{1}".format(quote_with_backticks(db1, server1_sql_mode),
+                                  quote_with_backticks(item[1][0],
+                                                       server1_sql_mode))
+        q_obj2 = "{0}.{1}".format(quote_with_backticks(db2, server2_sql_mode),
+                                  quote_with_backticks(item[1][0],
+                                                       server2_sql_mode))
 
         reporter.report_object(obj_type, item[1][0])
 
@@ -516,13 +522,16 @@ def compare_all_databases(server1_val, server2_val, exclude_list, options):
     }
     server1, server2 = connect_servers(server1_val, server2_val, conn_options)
 
-    # Check if the specified servers are the same.
-    if server1.port == server2.port and server1.is_alias(server2.host):
+    # Check if the specified servers are the same
+    if server2 is None or server1.port == server2.port and \
+            server1.is_alias(server2.host):
         raise UtilError(
             "Specified servers are the same (server1={host1}:{port1} and "
             "server2={host2}:{port2}). Cannot compare all databases on the "
             "same server.".format(host1=server1.host, port1=server1.port,
-                                  host2=server2.host, port2=server2.port))
+                                  host2=getattr(server2, "host", server1.host),
+                                  port2=getattr(server2, "port", server1.port))
+        )
 
     # Get all databases, except those used in --exclude
     get_dbs_query = """
@@ -531,6 +540,7 @@ def compare_all_databases(server1_val, server2_val, exclude_list, options):
         WHERE SCHEMA_NAME != 'INFORMATION_SCHEMA'
         AND SCHEMA_NAME != 'PERFORMANCE_SCHEMA'
         AND SCHEMA_NAME != 'mysql'
+        AND SCHEMA_NAME != 'sys'
         {0}"""
     conditions = ""
     if exclude_list:
