@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,10 +23,11 @@ import locale
 import os
 import sys
 
+from mysql.utilities.exception import UtilError
 from mysql.utilities.common.format import print_list
 from mysql.utilities.common.tools import encode
 from mysql.utilities.common.user import User
-from mysql.utilities.exception import UtilError
+
 
 # Constants
 _KB = 1024.0
@@ -136,7 +137,7 @@ def _get_db_dir_size(folder):
         return None
     for item in os.listdir(folder):
         name, ext = os.path.splitext(item)
-        if ext.upper() not in (".MYD", ".MYI") and \
+        if ext.upper() not in (".MYD", ".MYI", ".IBD") and \
            name.upper() not in ('SLOW_LOG', 'GENERAL_LOG'):
             itemfolder = os.path.join(folder, item)
             if os.path.isfile(itemfolder):
@@ -159,17 +160,17 @@ def _find_tablespace_files(folder, verbosity=0):
     try:
         for item in os.listdir(folder):
             itempath = os.path.join(folder, item)
-            name, ext = os.path.splitext(item)
+            _, ext = os.path.splitext(item)
             if os.path.isfile(itempath):
-                name, ext = os.path.splitext(item)
+                _, ext = os.path.splitext(item)
                 if ext.upper() == "IBD":
                     size = os.path.getsize(itempath)
                     total += size
                     if verbosity > 0:
-                        row = (item, size,'file tablespace', '')
+                        row = (item, size, 'file tablespace', '')
                     else:
                         row = (item, size)
-    
+
                     tablespaces.append(row)
             else:
                 subdir, tot = _find_tablespace_files(itempath, verbosity)
@@ -198,7 +199,7 @@ def _build_logfile_list(server, log_name, suffix='_file'):
     if res != [] and res[0][1].upper() == 'OFF':
         print "# The %s is turned off on the server." % log_name
     else:
-        res = server.show_server_variable(log_name+suffix)
+        res = server.show_server_variable(log_name + suffix)
         if res == []:
             raise UtilError("Cannot get %s_file setting." % log_name)
         log_path = res[0][1]
@@ -214,7 +215,7 @@ def _build_logfile_list(server, log_name, suffix='_file'):
     return None, 0, 0
 
 
-def _get_log_information(server, log_name, suffix='_file'):
+def _get_log_information(server, log_name, suffix='_file', is_remote=False):
     """Get information about a specific log.
 
     This method checks the system variable of the log_name passed to see if
@@ -225,15 +226,21 @@ def _get_log_information(server, log_name, suffix='_file'):
     log_name[in]      Variable name for the log (e.g. slow_query_log)
     suffix[in]        Suffix of log variable name (e.g. slow_query_log_file)
                       default = '_file'
+    is_remote[in]     True is a remote server
 
     returns (tuple) (log files, total size)
     """
+    if is_remote:
+        print("# {0} information not accessible from a remote host."
+              "".format(log_name))
+        return (None, 0,)
+
     res = server.show_server_variable(log_name)
     if res != [] and res[0][1].upper() == 'OFF':
         print "# The %s is turned off on the server." % log_name
     else:
         log_file, log_path, log_size = _build_logfile_list(server, log_name,
-                                                       suffix)
+                                                           suffix)
         if log_file is None or log_path is None or \
            not os.access(log_path, os.R_OK):
             print "# %s information is not accessible. " % log_name + \
@@ -257,7 +264,7 @@ def _build_log_list(folder, prefix):
     binlogs = []
     if prefix is not None:
         for item in os.listdir(folder):
-            name, ext = os.path.splitext(item)
+            name, _ = os.path.splitext(item)
             if name.upper() == prefix.upper():
                 itempath = os.path.join(folder, item)
                 if os.path.isfile(itempath):
@@ -292,7 +299,7 @@ def _build_innodb_list(per_table, folder, datadir, specs, verbosity=0):
     # Here, we want to capture log files as well as tablespace files.
     if specs is not None:
         for item in os.listdir(folder):
-            name, ext = os.path.splitext(item)
+            name, _ = os.path.splitext(item)
             # Check specification list
             for spec in specs:
                 parts = spec.split(":")
@@ -330,8 +337,9 @@ def _build_innodb_list(per_table, folder, datadir, specs, verbosity=0):
     return tablespaces, total_size
 
 
-def _build_db_list(server, rows, include_list, datadir, format=False,
-                   have_read=False, verbosity=0, include_empty=True):
+def _build_db_list(server, rows, include_list, datadir, fmt=False,
+                   have_read=False, verbosity=0, include_empty=True,
+                   is_remote=False):
     """Build a list of all databases and their totals.
 
     This method reads a list of databases and their calculated sizes
@@ -358,10 +366,11 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
     rows[in]          A list of databases and their calculated sizes
     include_list[in]  A list of databases included on the command line
     datadir[in]       The data directory
-    format[in]        If True, format columns and rows to standard sizes
+    fmt[in]           If True, format columns and rows to standard sizes
     have_read[in]     If True, user has read access to datadir path
     verbosity[in]     Controls how much data is shown
     include_empty[in] Include empty databases in list
+    is_remote[in]     True is a remote server
 
     return (tuple) (column headers, rows, total size)
     """
@@ -373,7 +382,7 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
     # build the list
     for row in rows:
         # If user can read the datadir, calculate actual and misc file totals
-        if have_read:
+        if have_read and not is_remote:
             # Encode database name (with strange characters) to the
             # corresponding directory name.
             db_dir = encode(row[0])
@@ -388,13 +397,13 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
             db_total = 0
         else:
             data_size = int(row[1])
-            db_total = int(row[1]) + misc_files
+            db_total = dbdir_size
 
         # Count total for all databases
-        total += data_size + misc_files
+        total += dbdir_size
 
-        if have_read:
-            if verbosity >= 2: # get all columns
+        if have_read and not is_remote:
+            if verbosity >= 2:  # get all columns
                 results.append((row[0], dbdir_size, data_size, misc_files,
                                 db_total))
             elif verbosity > 0:
@@ -404,14 +413,14 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
         else:
             results.append((row[0], db_total))
 
-    if have_read and verbosity > 0:
-        num_cols = min(verbosity+2, 4)
+    if have_read and not is_remote and verbosity > 0:
+        num_cols = min(verbosity + 2, 4)
     else:
         num_cols = 1
 
     # Build column list and format if necessary
     col_list = ['db_name']
-    if num_cols == 4: # get all columns
+    if num_cols == 4:  # get all columns
         col_list.append('db_dir_size')
         col_list.append('data_size')
         col_list.append('misc_files')
@@ -424,24 +433,24 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
         col_list.append('total')
 
     fmt_cols = []
-    max_col = [0,0,0,0]
-    if format:
+    max_col = [0, 0, 0, 0]
+    if fmt:
         fmt_cols.append(col_list[0])
-        for i in range(0,num_cols):
-            max_col[i] = _get_formatted_max_width(results, col_list, i+1)
-            fmt_cols.append("{0:>{1}}".format(col_list[i+1], max_col[i]))
+        for i in range(0, num_cols):
+            max_col[i] = _get_formatted_max_width(results, col_list, i + 1)
+            fmt_cols.append("{0:>{1}}".format(col_list[i + 1], max_col[i]))
     else:
         fmt_cols = col_list
 
     # format the list if needed
     fmt_rows = []
-    if format:
+    if fmt:
         for row in results:
-            fmt_data = ['','','','','']
+            fmt_data = ['', '', '', '', '']
             # Put in commas and justify strings
-            for i in range(0,num_cols):
-                fmt_data[i] = locale.format("%d", row[i+1], grouping=True)
-            if num_cols == 4: # get all columns
+            for i in range(0, num_cols):
+                fmt_data[i] = locale.format("%d", row[i + 1], grouping=True)
+            if num_cols == 4:  # get all columns
                 fmt_rows.append((row[0], fmt_data[0], fmt_data[1],
                                  fmt_data[2], fmt_data[3]))
             elif num_cols == 3:
@@ -454,33 +463,33 @@ def _build_db_list(server, rows, include_list, datadir, format=False,
 
     if include_empty:
         dbs = server.exec_query("SHOW DATABASES")
-        if len(fmt_rows) != len(dbs)-1:
+        if len(fmt_rows) != len(dbs) - 1:
             # We have orphaned database - databases not listed in IS.TABLES
             exclude_list = []
             for row in fmt_rows:
                 exclude_list.append(row[0])
             for db in dbs:
                 if db[0].upper() != "INFORMATION_SCHEMA" and \
-                   db[0] not in exclude_list and \
-                   (include_list is None or include_list == [] or \
-                    db[0] in include_list):
-                    if format:
-                        fmt_data = ['','','','','']
-                        for i in range(0,num_cols):
-                            if type(row[i+1]) == type(int):
+                        db[0] not in exclude_list and \
+                        (include_list is None or include_list == [] or
+                         db[0] in include_list):
+                    if fmt:
+                        fmt_data = ['', '', '', '', '']
+                        for i in range(0, num_cols):
+                            if type(row[i + 1]) == type(int):
                                 fmt_data[i] = locale.format("%s",
-                                                            int(row[i+1]),
+                                                            int(row[i + 1]),
                                                             grouping=True)
                             else:
-                                fmt_data[i] = locale.format("%s", row[i+1],
+                                fmt_data[i] = locale.format("%s", row[i + 1],
                                                             grouping=True)
-                        if num_cols == 4: # get all columns
+                        if num_cols == 4:  # get all columns
                             fmt_rows.insert(0, (db[0], fmt_data[0],
                                                 fmt_data[1], fmt_data[2],
                                                 fmt_data[3]))
                         elif num_cols == 3:
                             fmt_rows.insert(0, (db[0], fmt_data[0],
-                                             fmt_data[1], fmt_data[2]))
+                                                fmt_data[1], fmt_data[2]))
                         else:
                             fmt_rows.insert(0, (db[0], fmt_data[0]))
                     else:
@@ -508,13 +517,11 @@ def show_database_usage(server, datadir, dblist, options):
 
     returns True or exception on error
     """
-    
-    from mysql.utilities.common.format import print_list
-
-    format = options.get("format", "grid")
+    fmt = options.get("format", "grid")
     no_headers = options.get("no_headers", False)
     verbosity = options.get("verbosity", 0)
     have_read = options.get("have_read", False)
+    is_remote = options.get("is_remote", False)
     include_empty = options.get("do_empty", True)
     do_all = options.get("do_all", True)
     quiet = options.get("quiet", False)
@@ -528,9 +535,9 @@ def show_database_usage(server, datadir, dblist, options):
     if len(dblist) > 0:
         include_list = "("
         stop = len(dblist)
-        for i in range(0,stop):
+        for i in range(0, stop):
             include_list += "'%s'" % dblist[i]
-            if i < stop-1:
+            if i < stop - 1:
                 include_list += ", "
         include_list += ")"
         where_clause = "WHERE table_schema IN %s" % include_list
@@ -542,13 +549,14 @@ def show_database_usage(server, datadir, dblist, options):
 
     # Get list of databases with sizes and formatted when necessary
     columns, rows, db_total = _build_db_list(server, res, dblist, datadir,
-                                             format == "grid",
+                                             fmt == "grid",
                                              have_read, verbosity,
-                                             include_empty or do_all)
+                                             include_empty or do_all,
+                                             is_remote)
 
     if not quiet:
         print "# Database totals:"
-    print_list(sys.stdout, format, columns, rows, no_headers)
+    print_list(sys.stdout, fmt, columns, rows, no_headers)
     if not quiet:
         _print_size("\nTotal database disk usage = ", db_total)
         print
@@ -567,38 +575,36 @@ def show_logfile_usage(server, options):
 
     return True or raise exception on error
     """
-    from mysql.utilities.common.format import print_list
-
-    format = options.get("format", "grid")
+    fmt = options.get("format", "grid")
     no_headers = options.get("no_headers", False)
-    verbosity = options.get("verbosity", 0)
-    have_read = options.get("have_read", False)
+    is_remote = options.get("is_remote", False)
     quiet = options.get("quiet", False)
 
     if not quiet:
         print "# Log information."
     total = 0
-    
+
     _LOG_NAMES = [
         ('general_log', '_file'), ('slow_query_log', '_file'),
         ('log_error', '')
     ]
     logs = []
     for log_name in _LOG_NAMES:
-        log, size = _get_log_information(server, log_name[0], log_name[1])
+        (log, size,) = _get_log_information(server, log_name[0], log_name[1],
+                                            is_remote)
         if log is not None:
             logs.append((log, size))
         total += size
-    
+
     fmt_logs = []
     columns = ['log_name', 'size']
     if len(logs) > 0:
-        if format == 'grid':
+        if fmt == 'grid':
             max_col = _get_formatted_max_width(logs, columns, 1)
             if max_col < len('size'):
                 max_col = len('size')
             size = "{0:>{1}}".format('size', max_col)
-            columns = ['log_name',size]
+            columns = ['log_name', size]
             for row in logs:
                 # Add commas
                 size = locale.format("%d", row[1], grouping=True)
@@ -609,7 +615,7 @@ def show_logfile_usage(server, options):
         else:
             fmt_logs = logs
 
-        print_list(sys.stdout, format, columns, fmt_logs, no_headers)
+        print_list(sys.stdout, fmt, columns, fmt_logs, no_headers)
         if not quiet:
             _print_size("\nTotal size of logs = ", total)
             print
@@ -671,6 +677,8 @@ def show_log_usage(server, datadir, options):
     return True or raise exception on error
     """
     log_type = options.get("log_type", "binary log")
+    have_read = options.get("have_read", False)
+    is_remote = options.get("is_remote", False)
     quiet = options.get("quiet", False)
 
     # Check privileges to execute required queries: SUPER or REPLICATION CLIENT
@@ -687,7 +695,7 @@ def show_log_usage(server, datadir, options):
             print("# Binary logging is turned off on the server.")
             return True
         # Check required privileges according to the access to the datadir.
-        if os.access(datadir, os.R_OK):
+        if not is_remote and have_read:
             # Requires SUPER or REPLICATION CLIENT to execute:
             # SHOW MASTER STATUS.
             if not has_super and not has_rpl_client:
@@ -699,7 +707,7 @@ def show_log_usage(server, datadir, options):
             # Requires SUPER for server < 5.6.6 or also REPLICATION CLIENT for
             # server >= 5.6.6 to execute: SHOW BINARY LOGS.
             if (server.check_version_compat(5, 6, 6)
-                and not has_super and not has_rpl_client):
+               and not has_super and not has_rpl_client):
                 print("# {0} information not accessible. User must have the "
                       "SUPER or REPLICATION CLIENT "
                       "privilege.".format(log_type.capitalize()))
@@ -717,14 +725,13 @@ def show_log_usage(server, datadir, options):
                   "privilege.".format(log_type.capitalize()))
             return True
         # Can only retrieve usage information from the localhost filesystem.
-        if not os.access(datadir, os.R_OK):
-            if not server.is_alias("localhost"):
-                print("# {0} information not accessible from a remote "
-                      "host.".format(log_type.capitalize()))
-            else:
-                print("# {0} information not accessible. Check your "
-                      "permissions to {1}.".format(log_type.capitalize(),
-                                                   datadir))
+        if is_remote:
+            print("# {0} information not accessible from a remote host."
+                  "".format(log_type.capitalize()))
+            return True
+        elif not have_read:
+            print("# {0} information not accessible. Check your permissions "
+                  "to {1}.".format(log_type.capitalize(), datadir))
             return True
 
     # Check server status and availability of specified log file type.
@@ -755,7 +762,7 @@ def show_log_usage(server, datadir, options):
         print("# {0} information:".format(log_type.capitalize()))
         print("Current {0} file = {1}".format(log_type, current_log))
 
-    if log_type == 'binary log' and not os.access(datadir, os.R_OK):
+    if log_type == 'binary log' and (is_remote or not have_read):
         # Retrieve binlog usage info from SHOW BINARY LOGS.
         try:
             logs = server.exec_query("SHOW BINARY LOGS")
@@ -808,12 +815,10 @@ def show_innodb_usage(server, datadir, options):
 
     return True or raise exception on error
     """
-    from mysql.utilities.common.format import print_list
-
-    format = options.get("format", "grid")
+    fmt = options.get("format", "grid")
     no_headers = options.get("no_headers", False)
+    is_remote = options.get("is_remote", False)
     verbosity = options.get("verbosity", 0)
-    have_read = options.get("have_read", False)
     quiet = options.get("quiet", False)
 
     # Check to see if we have innodb
@@ -845,7 +850,7 @@ def show_innodb_usage(server, datadir, options):
     else:
         innodb_dir = datadir
 
-    if os.access(innodb_dir, os.R_OK):
+    if not is_remote and os.access(innodb_dir, os.R_OK):
         if not quiet:
             print "# InnoDB tablespace information:"
 
@@ -868,7 +873,7 @@ def show_innodb_usage(server, datadir, options):
             columns.append('specificaton')
         size = 'size'
         fmt_innodb = []
-        if format.upper() == 'GRID':
+        if fmt.upper() == 'GRID':
             max_col = _get_formatted_max_width(innodb, columns, 1)
             if max_col < len('size'):
                 max_col = len('size')
@@ -892,7 +897,7 @@ def show_innodb_usage(server, datadir, options):
         else:
             fmt_innodb = innodb
 
-        print_list(sys.stdout, format, columns, fmt_innodb, no_headers)
+        print_list(sys.stdout, fmt, columns, fmt_innodb, no_headers)
         if not quiet:
             _print_size("\nTotal size of InnoDB files = ", total)
             print
@@ -906,8 +911,8 @@ def show_innodb_usage(server, datadir, options):
                         print "Tablespace %s can be " % tablespace[3] + \
                               "extended by using %s:%sM[...]\n" % \
                               (parts[0], size)
-
-
+    elif is_remote:
+        print("# InnoDB data information not accessible from a remote host.")
     else:
         print "# InnoDB data file information is not accessible. " + \
               "Check your permissions."
