@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2014, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -72,6 +72,9 @@ PROCEDURE_TYPE = 'PROCEDURE'
 ROUTINE_TYPE = 'ROUTINE'
 FUNCTION_TYPE = 'FUNCTION'
 GLOBAL_TYPE = 'GLOBAL'
+GLOBAL_LEVEL = 3
+DATABASE_LEVEL = 2
+OBJECT_LEVEL = 1
 
 ALL_PRIVS_LOOKUP_DICT = {PROCEDURE_TYPE: _ROUTINE_ALL_PRIVS,
                          ROUTINE_TYPE: _ROUTINE_ALL_PRIVS,
@@ -92,11 +95,13 @@ def get_table_privs(server, db_name, table_name):
     Returns list of tuples (<Grantee>, <SET OF GRANTS>).
     """
     tpl_lst = []
+    # Get sql_mode in server
+    sql_mode = server.select_variable("SQL_MODE")
     # Remove backticks if necessary
-    if is_quoted_with_backticks(db_name):
-        db_name = remove_backtick_quoting(db_name)
-    if is_quoted_with_backticks(table_name):
-        table_name = remove_backtick_quoting(table_name)
+    if is_quoted_with_backticks(db_name, sql_mode):
+        db_name = remove_backtick_quoting(db_name, sql_mode)
+    if is_quoted_with_backticks(table_name, sql_mode):
+        table_name = remove_backtick_quoting(table_name, sql_mode)
 
     # Build query
     query = _TABLE_PRIV_QUERY.format(db_name, table_name)
@@ -123,9 +128,11 @@ def get_db_privs(server, db_name):
     Returns list of tuples (<Grantee>, <SET OF GRANTS>).
     """
     tpl_lst = []
+    # Get sql_mode in server
+    sql_mode = server.select_variable("SQL_MODE")
     # remove backticks if necessary
-    if is_quoted_with_backticks(db_name):
-        db_name = remove_backtick_quoting(db_name)
+    if is_quoted_with_backticks(db_name, sql_mode):
+        db_name = remove_backtick_quoting(db_name, sql_mode)
 
     # Build query
     query = _DB_PRIVS_QUERY.format(db_name)
@@ -175,11 +182,13 @@ def get_routine_privs(server, db_name, routine_name):
     Returns list of tuples (<GRANTEE>, <SET OF GRANTS>).
     """
     tpl_lst = []
+    # Get sql_mode in server
+    sql_mode = server.select_variable("SQL_MODE")
     # remove backticks if necesssary
-    if is_quoted_with_backticks(db_name):
-        db_name = remove_backtick_quoting(db_name)
-    if is_quoted_with_backticks(routine_name):
-        routine_name = remove_backtick_quoting(routine_name)
+    if is_quoted_with_backticks(db_name, sql_mode):
+        db_name = remove_backtick_quoting(db_name, sql_mode)
+    if is_quoted_with_backticks(routine_name, sql_mode):
+        routine_name = remove_backtick_quoting(routine_name, sql_mode)
 
     # Build query
     query = _PROCS_PRIV_QUERY.format(db_name, routine_name)
@@ -248,20 +257,22 @@ def filter_grants(grant_set, obj_type_str):
     return grant_set.intersection(all_privs_set)
 
 
-def _build_privilege_dicts(server, obj_type_dict):
+def _build_privilege_dicts(server, obj_type_dict, inherit_level=GLOBAL_LEVEL):
     """Builds TABLE, ROUTINE and DB dictionaries with grantee privileges
 
     server[in]        Server class instance
     obj_type_dict[in] dictionary with the list of objects to obtain the
                       grantee and respective grant information, organized
-                      by object type.
+                      by object type
+    inherit_level[in] Level of inheritance that should be taken into account.
+                      It must be one of GLOBAL_LEVEL, DATABASE_LEVEL or
+                      OBJECT_LEVEL
 
     This method builds and returns the 3 dictionaries with grantee
     information taking into account the grant hierarchy from mysql, i.e.
     global grants apply to all objects and database grants apply to all
     the database objects (tables, procedures and functions).
     """
-
     # Get the global Grants:
     global_grantee_lst = get_global_privs(server)
     # Build the Database level grants dict.
@@ -271,10 +282,12 @@ def _build_privilege_dicts(server, obj_type_dict):
         db_privs_lst = get_db_privs(server, db_name)
         for grantee, priv_set in db_privs_lst:
             db_grantee_dict[db_name][grantee] = priv_set
-        # Global privileges also apply to the database level.
-        for grantee, priv_set in global_grantee_lst:
-            db_grantee_dict[db_name][grantee].update(
-                filter_grants(priv_set, DATABASE_TYPE))
+        if inherit_level >= GLOBAL_LEVEL:
+            # If global inheritance level is turned on, global privileges
+            # also apply to the database level.
+            for grantee, priv_set in global_grantee_lst:
+                db_grantee_dict[db_name][grantee].update(
+                    filter_grants(priv_set, DATABASE_TYPE))
 
     # Build the table Level grants dict.
     # {db_name: {tbl_name: {grantee: set(privileges)}}}
@@ -286,24 +299,26 @@ def _build_privilege_dicts(server, obj_type_dict):
         for grantee, priv_set in tbl_privs_lst:
             table_grantee_dict[db_name][tbl_name][grantee] = priv_set
         # Existing db and global_grantee level privileges also apply to
-        # the table level.
-        # If we already have the privileges for the database where the
-        # table is at, we can use that information.
-        if db_grantee_dict[db_name]:
-            for grantee, priv_set in db_grantee_dict[db_name].iteritems():
-                table_grantee_dict[db_name][tbl_name][grantee].update(
-                    filter_grants(priv_set, TABLE_TYPE))
-        else:
-            # Get the grant information for the db the table is at and
-            # merge it together with global grants.
-            db_privs_lst = get_db_privs(server, db_name)
-            for grantee, priv_set in db_privs_lst:
-                table_grantee_dict[db_name][tbl_name][grantee].update(
-                    filter_grants(priv_set, TABLE_TYPE))
-            # Now do the same with global grants
-            for grantee, priv_set in global_grantee_lst:
-                table_grantee_dict[db_name][tbl_name][grantee].update(
-                    filter_grants(priv_set, TABLE_TYPE))
+        # the table level if inherit level is database level or higher
+        if inherit_level >= DATABASE_LEVEL:
+            # If we already have the privileges for the database where the
+            # table is at, we can use that information.
+            if db_grantee_dict[db_name]:
+                for grantee, priv_set in db_grantee_dict[db_name].iteritems():
+                    table_grantee_dict[db_name][tbl_name][grantee].update(
+                        filter_grants(priv_set, TABLE_TYPE))
+            else:
+                # Get the grant information for the db the table is at and
+                # merge it together with database grants.
+                db_privs_lst = get_db_privs(server, db_name)
+                for grantee, priv_set in db_privs_lst:
+                    table_grantee_dict[db_name][tbl_name][grantee].update(
+                        filter_grants(priv_set, TABLE_TYPE))
+                # Now do the same with global grants
+                if inherit_level >= GLOBAL_LEVEL:
+                    for grantee, priv_set in global_grantee_lst:
+                        table_grantee_dict[db_name][tbl_name][grantee].update(
+                            filter_grants(priv_set, TABLE_TYPE))
 
     # Build the ROUTINE Level grants dict.
     # {db_name: {proc_name: {user: set(privileges)}}}
@@ -314,24 +329,26 @@ def _build_privilege_dicts(server, obj_type_dict):
         for grantee, priv_set in proc_privs_lst:
             proc_grantee_dict[db_name][proc_name][grantee] = priv_set
         # Existing db and global_grantee level privileges also apply to
-        # the routine level.
-        # If we already have the privileges for the database where the
-        # routine is at, we can use that information.
-        if db_grantee_dict[db_name]:
-            for grantee, priv_set in db_grantee_dict[db_name].iteritems():
-                proc_grantee_dict[db_name][proc_name][grantee].update(
-                    filter_grants(priv_set, ROUTINE_TYPE))
-        else:
-            # Get the grant information for the db the routine belongs to
-            #  and merge it together with global grants.
-            db_privs_lst = get_db_privs(server, db_name)
-            for grantee, priv_set in db_privs_lst:
-                proc_grantee_dict[db_name][proc_name][grantee].update(
-                    filter_grants(priv_set, ROUTINE_TYPE))
-            # Now do the same with global grants.
-            for grantee, priv_set in global_grantee_lst:
-                proc_grantee_dict[db_name][proc_name][grantee].update(
-                    filter_grants(priv_set, ROUTINE_TYPE))
+        # the routine level if inherit level is database level or higher
+        if inherit_level >= DATABASE_LEVEL:
+            # If we already have the privileges for the database where the
+            # routine is at, we can use that information.
+            if db_grantee_dict[db_name]:
+                for grantee, priv_set in db_grantee_dict[db_name].iteritems():
+                    proc_grantee_dict[db_name][proc_name][grantee].update(
+                        filter_grants(priv_set, ROUTINE_TYPE))
+            else:
+                # Get the grant information for the db the routine belongs to
+                #  and merge it together with global grants.
+                db_privs_lst = get_db_privs(server, db_name)
+                for grantee, priv_set in db_privs_lst:
+                    proc_grantee_dict[db_name][proc_name][grantee].update(
+                        filter_grants(priv_set, ROUTINE_TYPE))
+                # Now do the same with global grants.
+                if inherit_level >= GLOBAL_LEVEL:
+                    for grantee, priv_set in global_grantee_lst:
+                        proc_grantee_dict[db_name][proc_name][grantee].update(
+                            filter_grants(priv_set, ROUTINE_TYPE))
 
     # TODO Refactor the code below to remove code repetition.
 
@@ -397,7 +414,8 @@ def _has_all_privileges(query_priv_set, grantee_priv_set, obj_type):
     return query_priv_set.issubset(grantee_priv_set)
 
 
-def get_grantees(server, valid_obj_type_dict, req_privileges=None):
+def get_grantees(server, valid_obj_type_dict, req_privileges=None,
+                 inherit_level=GLOBAL_LEVEL):
     """Get grantees and respective grants for the specified objects.
 
     server[in]            Server class instance
@@ -405,11 +423,14 @@ def get_grantees(server, valid_obj_type_dict, req_privileges=None):
                           by object type. We assume that each object exists
                           on the server
     req_privileges[in]    Optional set of required privileges
+    inherit_level[in]     Level of inheritance that should be taken into
+                          account. It must be one of GLOBAL_LEVEL,
+                          DATABASE_LEVEL or OBJECT_LEVEL
     """
 
     # Build the privilege dicts
     db_dict, table_dict, proc_dict = _build_privilege_dicts(
-        server, valid_obj_type_dict)
+        server, valid_obj_type_dict, inherit_level)
 
     # Build final dict with grantee/grant information, taking into account
     # required privileges
