@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -197,6 +197,29 @@ if __name__ == '__main__':
                            "values can be used separated by commas. The "
                            "default is health.")
 
+    # Add connection timeout for C/Py connections
+    #
+    # Note: we do not set a default here as we want C/Py to host the
+    # default. Utilities need not control the default, rather, we only
+    # want to provide an option for the user to set it should she have
+    # need to tune for a particular failover scenario. See BUG#22932375
+    # for more details.
+    parser.add_option("--connection-timeout", action="store", type="int",
+                      dest = "conn_timeout", default = None, help = "set the "
+                      "connection timeout for TCP and Unix socket "
+                      "connections for all master, slaves, and candidate "
+                      "slaves specified. Default is 10 as provided in the "
+                      "Connector/Python module.")
+
+    # Add master failover delay check
+    parser.add_option("--master-fail-retry", action="store", dest="fail_retry",
+                      type="int", default=None, help="time in seconds to wait "
+                      "to determine master is down. The failover check will "
+                      "be run again when the retry delay expires. Can be used "
+                      "to introduce a longer period between when master is "
+                      "detected as unavailable to declaring it down. This "
+                      "option is not used with --exec-fail-check. ")
+
     # Add verbosity mode
     add_verbosity(parser, False)
 
@@ -241,6 +264,14 @@ if __name__ == '__main__':
         parser.error("Failover mode = 'elect' requires at least one "
                      "candidate.")
 
+    if opt.fail_retry and opt.exec_fail:
+        parser.error("The --master-fail-retry option cannot be used "
+                     "with --exec-fail-check.")
+
+    if opt.fail_retry and opt.fail_retry < 1:
+        parser.error("The --master-fail-retry option must be a positive "
+                     "integer > 0.")
+
     # Parse the master, slaves, and candidates connection parameters
     try:
         master_val, slaves_val, candidates_val = parse_topology_connections(
@@ -285,6 +316,7 @@ if __name__ == '__main__':
         'pidfile': opt.pidfile,
         'report_values': opt.report_values,
         'script_threshold': opt.script_threshold,
+        'fail_retry': opt.fail_retry,
     }
 
     # Purge log file of old data
