@@ -33,7 +33,9 @@ from mysql.utilities.command.grep import ObjectGrep, OBJECT_TYPES
 from mysql.utilities.common.tools import check_connector_python
 from mysql.utilities.common.options import (add_regexp, setup_common_options,
                                             add_format_option,
-                                            add_character_set_option)
+                                            add_character_set_option,
+                                            get_ssl_dict,
+                                            check_password_security)
 from mysql.utilities.exception import UtilError
 
 # Check for connector/python
@@ -93,6 +95,9 @@ if __name__ == '__main__':
 
     options, args = parser.parse_args()
 
+    # Check security settings
+    check_password_security(options, args)
+
     _LOOKS_LIKE_CONNECTION_MSG = """Pattern '{pattern}' looks like a
     connection specification. Use --pattern if this is really what you
     want"""
@@ -125,8 +130,9 @@ if __name__ == '__main__':
         if options.print_sql:
             print(command.sql())
         else:
+            ssl_opts = get_ssl_dict(options)
             command.execute(options.server, format=options.format,
-                            charset=options.charset)
+                            charset=options.charset, ssl_opts=ssl_opts)
     except UtilError:
         _, err, _ = sys.exc_info()
         sys.stderr.write("ERROR: {0}\n".format(err.errmsg))
