@@ -35,7 +35,9 @@ from mysql.utilities.common.ip_parser import parse_connection
 from mysql.utilities.common.server import check_hostname_alias
 from mysql.utilities.common.tools import check_connector_python
 from mysql.utilities.common.options import (setup_common_options, add_rpl_user,
-                                            add_verbosity)
+                                            add_verbosity, add_ssl_options,
+                                            get_ssl_dict,
+                                            check_password_security)
 from mysql.utilities.common.messages import (PARSE_ERR_OPTS_REQ,
                                              WARN_OPT_USING_DEFAULT)
 
@@ -62,14 +64,16 @@ if __name__ == '__main__':
                       type="string", default=None,
                       help="connection information for master server in the "
                            "form: <user>[:<password>]@<host>[:<port>]"
-                           "[:<socket>] or <login-path>[:<port>][:<socket>].")
+                           "[:<socket>] or <login-path>[:<port>][:<socket>]"
+                           " or <config-path>[<[group]>].")
 
     # Connection information for the destination server
     parser.add_option('--slave', action="store", dest="slave",
                       type="string", default=None,
                       help="connection information for slave server in the "
                            "form: <user>[:<password>]@<host>[:<port>]"
-                           "[:<socket>] or <login-path>[:<port>][:<socket>].")
+                           "[:<socket>] or <login-path>[:<port>][:<socket>]"
+                           " or <config-path>[<[group]>].")
 
     # Replication user and password
     add_rpl_user(parser)
@@ -104,12 +108,18 @@ if __name__ == '__main__':
                            "in the binary logging of the master. Not valid "
                            "with --master-log-file or --master-log-pos.")
 
+    # Add ssl options
+    add_ssl_options(parser)
+
     # Add verbosity
     add_verbosity(parser)
 
     # Now we process the rest of the arguments.
     opt, args = parser.parse_args()
 
+    # Check security settings
+    check_password_security(opt, args)
+
     # option --master is required (mandatory)
     if not opt.master:
         default_val = 'root@localhost:3306'
@@ -122,6 +132,10 @@ if __name__ == '__main__':
     if not opt.slave:
         parser.error(PARSE_ERR_OPTS_REQ.format(opt='--slave'))
 
+    # option --rpl-user is required (mandatory)
+    if not opt.rpl_user:
+        parser.error(PARSE_ERR_OPTS_REQ.format(opt="--rpl-user"))
+
     # Parse source connection values
     try:
         m_values = parse_connection(opt.master, None, opt)
@@ -167,6 +181,9 @@ if __name__ == '__main__':
         'from_beginning': opt.from_beginning,
     }
 
+    # Add ssl Values to options instead of connection.
+    options.update(get_ssl_dict(opt))
+
     try:
         setup_replication(m_values, s_values, opt.rpl_user, options,
                           opt.test_db)
