@@ -32,8 +32,11 @@ from mysql.utilities.command.serverinfo import show_server_info
 from mysql.utilities.common.tools import check_connector_python
 from mysql.utilities.common.options import (add_basedir_option, add_verbosity,
                                             add_format_option,
-                                            check_basedir_option,
-                                            setup_common_options)
+                                            get_ssl_dict,
+                                            add_no_headers_option,
+                                            check_dir_option,
+                                            setup_common_options,
+                                            check_password_security)
 
 
 # Constants
@@ -56,10 +59,8 @@ if __name__ == '__main__':
     add_format_option(parser, "display the output in either grid (default), "
                       "tab, csv, or vertical format", "grid")
 
-    # Header row
-    parser.add_option("-h", "--no-headers", action="store_true",
-                      dest="no_headers", default=False,
-                      help="do not show column headers")
+    # No header option
+    add_no_headers_option(parser, restricted_formats=['grid', 'tab', 'csv'])
 
     # Show my.cnf values
     parser.add_option("-d", "--show-defaults", action="store_true",
@@ -101,8 +102,19 @@ if __name__ == '__main__':
     # Now we process the rest of the arguments.
     opt, args = parser.parse_args()
 
-    # Check the basedir option for errors (e.g., invalid path)
-    check_basedir_option(parser, opt.basedir)
+    # Check security settings
+    check_password_security(opt, args)
+
+    # The --basedir and --datadir options are only required if --start is used
+    # otherwise they are ignored.
+    if opt.start:
+        # Check the basedir option for errors (e.g., invalid path).
+        check_dir_option(parser, opt.basedir, '--basedir', check_access=True,
+                         read_only=True)
+
+        # Check the datadir option for errors.
+        check_dir_option(parser, opt.datadir, '--datadir', check_access=True,
+                         read_only=False)
 
     # Check start timeout for minimal value
     if int(opt.start_timeout) < 10:
@@ -136,6 +148,9 @@ if __name__ == '__main__':
     if opt.server is None:
         parser.error("You must specify at least one server.")
 
+    # add ssl options values.
+    options.update(get_ssl_dict(opt))
+
     try:
         show_server_info(opt.server, options)
     except UtilError:
